/******************************************************************
/The adaptation to MUSCLE environment is based on code published by
/David Corcoran. Thanks.
/See www.linuxnet.com
/******************************************************************

/*****************************************************************
/ File   :   IFD_Handler.c
/ Author :   Fernando Llobregat
/ Date   :   Stand Alone version Oct-Nov, 1998/ Adapted to MUSCLE PC/SC June,1999
/ Purpose:   PC/SC compliant IFD_Handler for DeLaRue readers
/ License:   See file LICENSE
/                                  MALOUISWITHME
******************************************************************

******************************************************************
IMPORTANT:
This code is not at any point supported or published by DeLaRue, please send me directly any comments, bugs you find, and an e-mail just saying you are using it. This last point is important to put more effort in this project and show people linux gets important
e-mail: fernando.llobregat@es.delarue.com
************************************************************************/
#define USE_C_CONSTRUCTS

#include <stdio.h>
#include "ctapi.h"
#include <IFD_Handler.h>
#include <string.h>
#include <pcscdefines.h>

#define VendorName "Linux driver for DLR Readers-99\n"
#define IFDType "DE122/DE132\n"			
#define IFDVersion 0x01001111		


RESPONSECODE IO_Create_Channel ( DWORD ChannelId ) {

  RESPONSECODE lRetVal = IFD_SUCCESS;

  int iPort;
  unsigned int iTerm = 0;

  unsigned short Unibble;         // Upper 16bits of DWORD. (DDDD)
  unsigned short Lnibble;         // Lower 16bits of DWORD. (CCCC)

  Unibble = ChannelId >> 16;                 // Shift right 16 bits.
  Lnibble = ChannelId - ( Unibble << 16 );   // Shift left and subtract.

  switch ( Unibble ) {

    case 0x01:                               // Serial Port.

      switch ( Lnibble ) {

        case 0x3F8:
          iPort = PORT_COM1;
          break;

        case 0x2F8:
          iPort = PORT_COM2;
          break;

        case 0x3E8:
          iPort = PORT_COM3;
          break;

        case 0x2E8:
          iPort = PORT_COM4;
          break;
    
        default:
          lRetVal = IFD_NOT_SUPPORTED;
          break;
      }


      break;

    case 0x02:                               // Parallel Port.
      lRetVal = IFD_NOT_SUPPORTED;
      break;

    case 0x04:                               // PS 2 Port.
      lRetVal = IFD_NOT_SUPPORTED;
      break;

    case 0x08:                               // SCSI Port.
      lRetVal = IFD_NOT_SUPPORTED;
      break;

    case 0x10:                               // IDE Port.
      lRetVal = IFD_NOT_SUPPORTED;
      break;

    case 0x20:                               // USB Port.
      lRetVal = IFD_NOT_SUPPORTED;
      break;

    /* case 0xFy where y is vendor defined 0 - F is NOT implemented */

    default:                                 // Port Not Found.
      lRetVal = IFD_NOT_SUPPORTED;
      break;

  }


  if ( lRetVal == IFD_SUCCESS ) {
    
    if ( CT_init( iTerm, iPort ) == OK ) {
      lRetVal = IFD_SUCCESS;

     } else {
      lRetVal = IFD_COMMUNICATION_ERROR;
     }
  }


  return lRetVal;

}

RESPONSECODE IO_Close_Channel () {

  RESPONSECODE lRetVal = IFD_SUCCESS;

  unsigned int iTerm = 0;

  if ( CT_close( iTerm ) == OK ) {
    
    lRetVal = IFD_SUCCESS;

  } else {
       
    lRetVal = IFD_COMMUNICATION_ERROR;
  }

  return lRetVal;

}


RESPONSECODE IFD_Get_Capabilities ( DWORD Tag, BYTE Value[] ) {


  RESPONSECODE lRetVal;
  DWORD HighNibble;
  DWORD LowNibble;
  Device.Vendor_Name= VendorName;      	//Tag0x0100
  Device.IFD_Type= IFDType;		//Tag0x0101
  Device.IFD_Version= (DWORD)IFDVersion;	//Tag0x0102


  HighNibble = Tag >> 8;
  LowNibble  = Tag - ( HighNibble << 8 );   // Shift left and subtract.

  lRetVal=IFD_SUCCESS;
  if (HighNibble==0x01){

    switch (LowNibble){
    case (0x00):
      memcpy(Value,Device.Vendor_Name,strlen(Device.Vendor_Name)+1);
      break;
    case (0x01):
      memcpy(Value,Device.IFD_Type,strlen(Device.IFD_Type)+1);
      break;
    case (0x02):

      Value[3]=Device.IFD_Version; 
      Value[2]=Device.IFD_Version>>8;
      Value[1]=Device.IFD_Version>>16;
      Value[0]=Device.IFD_Version>>24;
      Value[4]='\n';
      break;
    }

  }


  else if ( HighNibble == 0x02 ) {

    switch( LowNibble ) {
      case 0x01:
        lRetVal = IFD_SUCCESS;  
        break;

    }

  } else if ( HighNibble == 0x03 ) {      /* This is the ICC_STATE */

    switch( LowNibble ) {
   
      case 0x00:
	break;
      case 0x01:

	
	if ( IFD_Is_ICC_Present() == IFD_SUCCESS ) {
          *(DWORD*)Value = IFD_ICC_PRESENT;
        } else {
          *(DWORD*)Value = IFD_ICC_NOT_PRESENT;
        }

        lRetVal = IFD_SUCCESS;  
		
	break;
      case 0x03:

        memcpy(Value, ICC.ATR, MAX_ATR_SIZE);
        lRetVal = IFD_SUCCESS;  
	
	break;
      case 0x04:
	break;


      default:
 	break;

    }

  }


  return lRetVal;
  
}

RESPONSECODE IFD_Set_Capabilities ( DWORD Tag, BYTE Value[] ) {

  RESPONSECODE lRetVal;
  DWORD HighNibble;
  DWORD LowNibble;

  HighNibble = Tag >> 8;
  LowNibble  = Tag - ( HighNibble << 8 );   // Shift left and subtract.

  if ( HighNibble == 0x03 ) {      /* This is the ICC_STATE */

    switch( LowNibble ) {
   
      case 0x00:
	break;
      case 0x01:
	break;
      case 0x03:
	break;
      case 0x04:
	break;

      /* This is an extension to the specification. Please see it's
         definition in the header file.                             */

      case 0x50:
	//ICC.ICC_LOCK_ID = *(DWORD*)Value; 
        lRetVal = IFD_SUCCESS;  
	break;

      case 0x51:
	//ICC.ICC_LOCK = *(DWORD*)Value; 
        lRetVal = IFD_SUCCESS;  
	break;

      default:
 	break;

    }

  }
  
  return lRetVal;

}

RESPONSECODE IFD_Set_Protocol_Parameters ( DWORD ProtocolType, 
					   BYTE SelectionFlags,
					   BYTE PTS1, BYTE PTS2,
					   BYTE PTS3 ) {

  RESPONSECODE lRetVal;
  
  return lRetVal;

}

RESPONSECODE IFD_Power_ICC ( DWORD ActionRequested ) {



  RESPONSECODE lRetVal;

  
  int i;
  int lc;
  unsigned int lr;
  unsigned int iTerm;
  
  BYTE sad;
  BYTE dad;
  BYTE Response[255];
  BYTE pcPowerUp[] = {0,0x60,0,0,0};
  BYTE pcPowerDown[] = {0,0x90,0x10,0,0};


  iTerm = 1;
  sad = 2;
  dad = 0;

  lc = 5;
  lr = 2;



  switch( ActionRequested) {

    case IFD_POWER_UP:
    
    lr = MAX_ATR_SIZE;
    if ( CT_data( iTerm, &dad, &sad, lc, pcPowerUp, &lr, Response ) ==
	 OK ) {
      //Hay que comprobar antes que tenemos una tarjeta en el lector.
      if ( (Response[0]==0x90) & (Response[1]==0xFF)){
	//Se ha producido un time-out de la tarjeta
	lRetVal=IFD_RESPONSE_TIMEOUT;
	return lRetVal;
      }

        memcpy( ICC.ATR, Response, lr );          // Copies the ATR.           
	//Respetamos el 90:00 que vienes despues del ATR
        for ( i=lr; i < MAX_ATR_SIZE; i++ ) {   // Copies zeros to rest.
	  ICC.ATR[i] = 0x00;
        }
	
        lRetVal = IFD_SUCCESS;      
      } else {
        lRetVal = IFD_ERROR_POWER_ACTION;
      }
    break;
    
  case IFD_POWER_DOWN:

       
    if ( CT_data( iTerm, &dad, &sad, lc, pcPowerDown, &lr, Response ) == 
	 OK ) {
     
      lRetVal = IFD_SUCCESS;
      
    } else {
      lRetVal = IFD_ERROR_POWER_ACTION;
    }
    break;

  case IFD_RESET:

      lRetVal = IFD_NOT_SUPPORTED;
      
    break;
  }
  return lRetVal;
}



RESPONSECODE IFD_Swallow_ICC() {
  
  RESPONSECODE lRetVal;
  lRetVal = IFD_ERROR_NOT_SUPPORTED;        // This is not supported.
  return lRetVal;

}

RESPONSECODE IFD_Eject_ICC() {
  
  RESPONSECODE lRetVal;
  lRetVal = IFD_ERROR_NOT_SUPPORTED;        // This is not supported.
  return lRetVal;

}

RESPONSECODE IFD_Confiscate_ICC() {
  
  RESPONSECODE lRetVal;
  lRetVal = IFD_ERROR_NOT_SUPPORTED;        // This is not supported.
  return lRetVal;

}

  RESPONSECODE IFD_Transmit_to_ICC ( 
                                     struct SCARD_IO_HEADER SendPci, 
                                     BYTE CommandData[],
                                     DWORD CommandSize, 
                                     BYTE ResponseData[], 
                                     DWORD *ResponseSize, 
                                     struct SCARD_IO_HEADER *RecvPci  
                                   )

{
  RESPONSECODE lRetVal;
  DWORD lc;
  DWORD Protocol;
  unsigned int lr;
  unsigned int iTerm;  
  BYTE sad,dad;
  
  iTerm = 1;
  sad = 2;
  dad = 0;
  lc=CommandSize;     

      if ( CT_data( iTerm, &dad, &sad, lc, CommandData, &lr, ResponseData ) ==
       OK ) {
  
    *ResponseSize=lr;

    lRetVal = IFD_SUCCESS;
    
    } else {
      lRetVal = IFD_COMMUNICATION_ERROR;
    }
    
  return lRetVal;
    
}
    
RESPONSECODE IFD_Is_ICC_Present() {


  RESPONSECODE TestRsp;
  
  unsigned int lc;
  unsigned  int lr;
  unsigned int iTerm;  
  int i;
  BYTE sad;
  BYTE dad;
  BYTE Response[255];
  BYTE pcStatus[5] = {0x0, 0x60, 0x10, 0x00, 0x00};

  iTerm = 0;

  i=0;
  sad = 2;
  dad = 0;
  lc = 5;
  lr = 0;

    TestRsp = CT_data( iTerm, &dad, &sad, lc, pcStatus, &lr, Response );

     if ( TestRsp == OK ) {

	 if ( Response[0]==0x60 && Response[1]==0x84 ) {  /* Card Inserted */


	   return IFD_ICC_PRESENT;

	 } else {

	   return IFD_ICC_NOT_PRESENT;
	 }
     }   

  return IFD_COMMUNICATION_ERROR;

}

RESPONSECODE IFD_Is_ICC_Absent() {

  RESPONSECODE lRetVal;

  int lc;
  unsigned int lr;
  unsigned int iTerm;  

  BYTE sad;
  BYTE dad;
  BYTE Response[255];
  BYTE pcStatus[5] = {0x00, 0x60, 0x10, 0x00, 0x00};

  iTerm = 0;

  sad = 2;
  dad = 0;

  lc = 5;
  lr = 7; 
   
    if ( CT_data( iTerm, &dad, &sad, lc, pcStatus, &lr, Response ) ==
	 OK ) {

	 if ( Response[0]==0x60 && Response[1]==0x81 ) {  /* Card Removed */

	   lRetVal = IFD_ICC_NOT_PRESENT;

	 } else {
	   lRetVal = IFD_ICC_PRESENT;
	 }
     }   
  
  return lRetVal;
  
}






























