/*****************************************************************
/
/ File   :   ctapi.c
/ Author :   David Corcoran
/ Date   :   September 2, 1998
/ Purpose:   Defines CT-API functions
/ License:   See file LICENSE
/
******************************************************************/
/*MODIFIED BY:
/Author:      Fernando Llobregat
/Date:        September 12, 1998
/Purpose:     CT-API function for DeLaRue readers
/License:     See file LICENSE
                      MALOUISWITHME
******************************************************************/
#include <stdio.h>
#include "defines.h"
#include "ctapi.h"
#include "serial.h"

int  CT_init ( unsigned int Ctn,  unsigned int pn ) {
  
  BOOL BretVal;        /* Return value from IO_InitializePort() */
  int IretVal;         /* Return for this function */  
  Ctn=0;
  



  switch( pn ) {
  case PORT_COM1:
    BretVal = IO_InitializePort(9600,8,'N',"/dev/ttyS0");
    break;
  case PORT_COM2:
    BretVal = IO_InitializePort(9600,8,'N',"/dev/ttyS1");
    break;
  case PORT_COM3:
    BretVal = IO_InitializePort(9600,8,'N',"/dev/ttyS2");
    break;
  case PORT_COM4:
    BretVal = IO_InitializePort(9600,8,'N',"/dev/ttyS3");
    break;
  default:
    BretVal = IO_InitializePort(9600,8,'N',"/dev/smartcard");
    break;
  }
  
  if (!BretVal) {
    IretVal = ERR_MEMORY;        /* Could not allocate port */
  } else {
    IretVal = OK;
  }
  
  return IretVal;  
}

int  CT_close( unsigned int Ctn ) {

  Ctn=0; 
  
  if (IO_Close() == TRUE) {
    return OK;
    
  } else {
    return ERR_CT;
    
  }
}            

int  CT_data( unsigned int ctn, unsigned char *dad, unsigned char *sad,
              unsigned int  lc, unsigned char *cmd, unsigned int  *lr,
              unsigned char *rsp ) {
  
  

  BYTE BoutComm[255];    /* Command Buffer  */

  BYTE BrdrResp[3];                  /* Reader Response */
  int IretVal;                       /* Return Value    */
  unsigned int i;

             /* This set's the reader mode to T=0 */
             /* It must be done here since no     */
             /* protocol has been negotiated yet  */
  /*Not needed for DLR reader*/


  ctn=0;
  
 IretVal = OK;
 if ( IretVal == OK ) {
  
   if ( *dad == 0 ) {      /* This command goes to the card */
    
    *sad = 0;  /* Source Smartcard */
    *dad = 2;  /* Destination Host */
    
    BoutComm[0] = 0x15;        /* Send Reader Block for DLR reader*/
    BoutComm[1] = 0x00;
    BoutComm[2] = lc;          /* Length of command */
    
    memcpy( &BoutComm[3], cmd, lc );  /* Copy the command */
    
    for ( i=0; i < lc+3; i++ ) {
      if ( !IO_Write( BoutComm[i] ) ) {
	IretVal = ERR_TRANS;          /* Transmission Error */
	break;
      }
      else {
	IretVal = OK;                 /* Transmission OK */
      }
    }
    
    IO_UpdateReturnBlock(5);
    
    if ( IretVal == OK ) {                /* Transmission was OK */
      	
	  if ( IO_Read( 3, BrdrResp ) ) {  /* Response Block/Size */
	    if ( BrdrResp[0] == 0x15 ) {   /* Sending Block       */
	      *lr = BrdrResp[2];           /* Length of Return    */
	      if ( IO_Read( *lr, rsp ) ) { /* Gets Response       */
		IretVal = OK;
	      } else {
		IretVal = ERR_TRANS;       /* Transmission Error */
	      }          
	    } else {
	      IretVal = ERR_INVALID;     /* Invalid Command    */
	    } 
	  } else {
	    IretVal = ERR_TRANS;         /* Transmission Error */

	  }
	
    }
  } else {
    IretVal = ERR_INVALID;              /* Invalid Destination */
  }
 }  

  if (IretVal != OK) {
    *lr = 0;
  }

for (i=0;i<lc;i++)
  printf("%x:",cmd[i]);

printf("\n");
for (i=0;i<*lr;i++)
  printf("%x:",rsp[i]);
printf("\n");

  return IretVal;
}

/* End of ctapi.c */

